@@ -1,12 +1,45 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function, division, absolute_import, unicode_literals
 
-from datetime import datetime
+from datetime import datetime, timedelta
 import re
 import logging
 
 import six
 
+keywords = {
+    'de': {
+        'now': ['jetzt', 'sofort', 'nun'],
+        'noon': ['mittag'],
+        'midnight': ['mitternacht'],
+        'today': ["heute"],
+        'tomorrow': ["morgen"],
+        'at': ['um', 'am'],
+        'days': [r'in (\d+) tagen'],
+        'weekdays': ["montag", "dienstag", "mittwoch", "donnerstag", "freitag", "samstag", "sonntag"],
+    },
+    'en': {
+        'now': ['now', 'right now', 'immediately'],
+        'noon': ['noon'],
+        'midnight': ['midnight'],
+        'today': ["today"],
+        'tomorrow': ["tomorrow"],
+        'at': ['at'],
+        'days': [r'in (\d+) days'],
+        'weekdays': ["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"],
+    },
+    'fr': {
+        'now': ['maitenant'],
+        'noon': ['midi'],
+        'midnight': ['minuit'],
+        'today': ["aujourd'hui"],
+        'tomorrow': ["demain"],
+        'days': [r"dans (\d+) jours"],
+        'at': [],  # TODO: "à" clashes with top level keywords
+        'weekdays': ["lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi", "dimanche"],
+    },
+}
+
 
 def _process_tokens(tokens, sloppy_validation=False):
     """Parse input tokens.
@@ -85,7 +118,9 @@ def _process_tokens(tokens, sloppy_validation=False):
 def _detect_language(keyword_dicts, tokens):
     """Detect the language of the tokens by finding the highest intersection
     with the keywords of a specific language."""
-    intersection_count = lambda a, b: len(set(a).intersection(b))
+
+    def intersection_count(a, b):
+        return len(set(a).intersection(b))
 
     counts = []
     for lang, keywords in keyword_dicts.items():
@@ -96,12 +131,69 @@ def _detect_language(keyword_dicts, tokens):
     return language
 
 
-def _parse_time(timestring, language):
+def _parse_date(datestring, keywords):
+    """Parse date tokens.
+
+    Args:
+        datestring: String containing a date specification.
+        keywords: Language keywords
+
+    Returns:
+        date string.
+
+    Raises:
+        ValueError: If time could not be parsed.
+
+    """
+    date = None
+    days_shift = None
+    # Keywords
+    for i, d in enumerate(["today", "tomorrow"]):
+        if any([t in datestring for t in keywords[d]]):
+            days_shift = i
+    # Weekdays
+    for i, d in enumerate(keywords["weekdays"]):
+        if d in datestring:
+            days_shift = i - datetime.now().weekday()
+            if days_shift <= 0:
+                days_shift += 7
+    # Shifts
+    if days_shift is None:
+        for pattern in keywords["days"]:
+            days_re = re.search(pattern, datestring)
+            if days_re:
+                try:
+                    days_shift = int(days_re.group(1))
+                except:
+                    pass
+
+    if days_shift is not None:
+        return datetime.now() + timedelta(days=days_shift)
+
+    # Regular date strings
+    for dateformat in [[r"(\d{2}/\d{2}/\d{4})", "%d/%m/%Y"], [r"(\d{2}/\d{2})", "%d/%m"]]:
+        days_re = re.search(dateformat[0], datestring)
+        if days_re:
+            try:
+                date = datetime.strptime(days_re.group(1), dateformat[1])
+            except:
+                continue
+            if date.year == 1900:
+                date = date.replace(year=datetime.now().year)
+            break
+
+    if date is not None:
+        return date.strftime("%Y/%m/%d")
+
+    return None
+
+
+def _parse_time(timestring, keywords):
     """Parse time tokens.
 
     Args:
         timestring: String containing a time specification.
-        language: The language string (e.g. 'en' or 'de').
+        keywords: Language keywords
 
     Returns:
         Time string.
@@ -111,49 +203,23 @@ def _parse_time(timestring, language):
 
     """
 
-    keywords = {
-        'de': {
-            'now': ['jetzt', 'sofort', 'nun'],
-            'noon': ['mittag'],
-            'midnight': ['mitternacht'],
-            'at': ['um', 'am'],
-        },
-        'en': {
-            'now': ['now', 'right now', 'immediately'],
-            'noon': ['noon'],
-            'midnight': ['midnight'],
-            'at': ['at'],
-        },
-        'fr': {
-            'now': ['maitenant'],
-            'noon': ['midi'],
-            'midnight': ['minuit'],
-            'at': [],  # TODO: "à" clashes with top level keywords
-        },
-    }
-
-    try:
-        kws = keywords[language]
-    except IndexError:
-        raise ValueError('Invalid language: "%s"!' % language)
-
     # Ignore "at" keywords
-    if timestring.split(' ', 1)[0] in kws['at']:
+    if timestring.split(' ', 1)[0] in keywords['at']:
         timestring = timestring.split(' ', 1)[1]
 
     # Parse regular time strings
-    regular_time_match = re.match(r'([0-2]?[0-9])[:\-\. ]([0-9]{2})', timestring)
+    # regular_time_match = re.search(r'([0-2]?[0-9])[:\-\. ]([0-9]{2})', timestring)
+    regular_time_match = re.search(r'(?<!/)(\d{2})(?::*)(\d{2})', timestring)
     if regular_time_match:
         return ':'.join(regular_time_match.groups())
-
-    if timestring.lower() in kws['now']:
+    timestring = timestring.lower()
+    if timestring in keywords['now']:
         return datetime.now().strftime('%H:%M')
-    if timestring.lower() in kws['noon']:
+    if timestring in keywords['noon']:
         return '12:00'
-    if timestring.lower() in kws['midnight']:
+    if timestring in keywords['midnight']:
         return '23:59'  # '00:00' would be the first minute of the day, not the last one.
-
-    raise ValueError('Time string "%s" could not be parsed.' % timestring)
+    raise ValueError('Time is missing or could not be parsed')
 
 
 def parse_input(tokens):
@@ -177,35 +243,25 @@ def parse_input(tokens):
             departure *and* arrival time are specified.
 
     """
+
     # Process tokens, get data dict and language
     data, language = _process_tokens(tokens)
-
+    if data == {}:
+        return data, language
+    try:
+        kws = keywords[language]
+    except IndexError:
+        raise ValueError('Invalid language: "%s"!' % language)
     # Map keys
-    if 'departure' in data:
-        data['time'] = _parse_time(data['departure'], language)
-        del data['departure']
-    if 'arrival' in data:
-        data['time'] = _parse_time(data['arrival'], language)
-        data['isArrivalTime'] = 1
-        del data['arrival']
+    for t in ["departure", "arrival"]:
+        if t in data:
+            data["time"] = _parse_time(data[t], kws)
+            date = _parse_date(data[t], kws)
+            if date is not None:
+                data["date"] = date
+            if t == "arrival":
+                data['isArrivalTime'] = 1
+            del data[t]
 
     logging.debug('Data: ' + repr(data))
     return data, language
-
-
-    """
-    transport.opendata.ch request params:
-    x from
-    x to
-    x via
-    - date
-    x time
-    x isArrivalTime
-    - transportations
-    - limit
-    - page
-    - direct
-    - sleeper
-    - couchette
-    - bike
-    """
